import { CookieSerializeOptions } from "cookie";
import { GetServerSidePropsContext, NextPageContext } from "next";
import { Except, JsonValue, SetRequired } from "type-fest";
export declare type Cookies = Record<string, JsonValue>;
export declare type CookieContext = SetRequired<Partial<GetServerSidePropsContext | NextPageContext>, "req" | "res">;
/**
 * An isomorphic class to set and get compressed state cookies.
 */
export declare class StateCookies {
    protected _allNames: string[];
    private readonly _context?;
    private _cookies?;
    /**
     * @param context The Next.js context for the request (if on the server)
     */
    constructor(context?: CookieContext);
    /**
     * Set the names of all the cookies that shall be parsed via `getAll()`
     */
    setAllNames(names: string[]): void;
    getAll(): Record<string, JsonValue>;
    set(name: string, state: any, options?: Except<CookieSerializeOptions, "encode" | "httpOnly">): void;
    private _encodeState;
    private _decodeState;
}
