import isEqual from 'fast-deep-equal';
import { HYDRATE } from 'next-redux-wrapper';
import { compressToEncodedURIComponent, decompressFromEncodedURIComponent } from 'lz-string';
import { parseCookies, setCookie } from 'nookies';
import { produce } from 'immer';
import get from 'lodash-es/get';
import set from 'lodash-es/set';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _createForOfIteratorHelperLoose(o, allowArrayLike) {
  var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"];
  if (it) return (it = it.call(o)).next.bind(it);

  if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
    if (it) o = it;
    var i = 0;
    return function () {
      if (i >= o.length) return {
        done: true
      };
      return {
        done: false,
        value: o[i++]
      };
    };
  }

  throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

/**
 * Given a `NextReduxCookieMiddlewareConfig` object, returns the corresponding list of
 * `SubtreeConfig` objects.
 */
function processMiddlewareConfig(config) {
  // Set defaults and destructure the config object
  var _ignoreStateFromStati = _extends({
    ignoreStateFromStaticProps: true,
    path: "/",
    sameSite: true
  }, config),
      subtrees = _ignoreStateFromStati.subtrees,
      globalSubtreeConfig = _objectWithoutPropertiesLoose(_ignoreStateFromStati, ["subtrees"]); // Turn strings into `SubtreeConfig` objects, set a default for the cookieName option, and apply
  // the global default config


  return subtrees.map(function (current) {
    if (typeof current === "string") {
      return _extends({
        subtree: current,
        cookieName: current
      }, globalSubtreeConfig);
    } // `current` is a `SubtreeConfig` object


    return _extends({}, globalSubtreeConfig, {
      cookieName: current.subtree
    }, current);
  });
}

var isClient = function isClient() {
  return typeof window !== "undefined";
};
/**
 * Returns whether the provided context is a `GetServerSidePropsContext` or a `PageContext` with
 * `req` and `res` set.
 */

var isSsrContext = function isSsrContext(context) {
  return context.req !== undefined && context.res !== undefined;
};
/**
 * Returns whether the provided context is an `AppContext`.
 */

var isAppContext = function isAppContext(context) {
  return context.ctx !== undefined;
};

/**
 * An isomorphic class to set and get compressed state cookies.
 */

var StateCookies = /*#__PURE__*/function () {
  /**
   * @param context The Next.js context for the request (if on the server)
   */
  function StateCookies(context) {
    this._allNames = [];
    this._context = context;
  }
  /**
   * Set the names of all the cookies that shall be parsed via `getAll()`
   */


  var _proto = StateCookies.prototype;

  _proto.setAllNames = function setAllNames(names) {
    this._allNames = names;
  };

  _proto.getAll = function getAll() {
    // Parse cookies if they have not been parsed, always re-parse cookies on the client
    if (typeof this._cookies === "undefined" || isClient()) {
      this._cookies = {};

      for (var _i = 0, _Object$entries = Object.entries(parseCookies(this._context, {
        decode: function decode(value) {
          return value;
        }
      })); _i < _Object$entries.length; _i++) {
        var _Object$entries$_i = _Object$entries[_i],
            name = _Object$entries$_i[0],
            value = _Object$entries$_i[1];

        if (this._allNames.includes(name)) {
          this._cookies[name] = this._decodeState(value);
        }
      }
    }

    return this._cookies;
  };

  _proto.set = function set(name, state, options) {
    setCookie(this._context, name, state, _extends({}, options, {
      encode: this._encodeState,
      httpOnly: false
    }));
  };

  _proto._encodeState = function _encodeState(state) {
    return compressToEncodedURIComponent(JSON.stringify(state));
  };

  _proto._decodeState = function _decodeState(state) {
    return JSON.parse(decompressFromEncodedURIComponent(state));
  };

  return StateCookies;
}();

/**
 * Given a list of subtree configurations, two state objects `stateA` and `stateB`, and a walker
 * function, invokes the walker function for each `SubtreeConfig` object. The walker function
 * receives a subtree config and the respective state subtrees from `stateA` and `stateB`. If a
 * subtree does not exist in a state object, `undefined` is passed instead.
 *
 * @param subtrees A list of `SubtreeConfig` objects for the walker function
 * @param stateA First state object
 * @param stateB Optional second state object. If omitted the walker function will always receive
 * `undefined` as its last parameter.
 * @param walker A function that will be invoked for each `SubtreeConfig` object from `subtrees`
 * with the `SubtreeConfig` object and the respective state subtrees from `stateA` and `stateB`.
 *
 * @returns A copy of `stateA` where those subtrees for which the walker function has returned a
 * value have been replaced by that value.
 */

var walkState = function walkState(subtrees, walker, stateA, stateB) {
  return produce(stateA, function (draftState) {
    for (var _iterator = _createForOfIteratorHelperLoose(subtrees), _step; !(_step = _iterator()).done;) {
      var subtreeConfig = _step.value;
      var subtree = subtreeConfig.subtree;
      var result = walker(subtreeConfig, get(stateA, subtree), get(stateB, subtree));

      if (typeof result !== "undefined") {
        set(draftState, subtree, result);
      }
    }
  });
};

/**
 * An action type that is dispatched internally on the server to pass the Next.js context to the
 * {@link nextReduxCookieMiddleware}.
 */

var SERVE_COOKIES = "next-redux-cookie-wrapper/SERVE_COOKIES";
/**
 * A tiny wrapper for the `makeStore()` function. It sends the received Next.js context to
 * {@link nextReduxCookieMiddleware} on the server via a {@link SERVE_COOKIES} action.
 *
 * Use it like this:
 * ```ts
 * export const wrapper = createWrapper<AppStore>(wrapMakeStore(makeStore));
 * ```
 *
 * @param makeStore The `makeStore()` function for `next-redux-wrapper`
 * @returns A wrapped version of the `makeStore()` function
 */

var wrapMakeStore = function wrapMakeStore(makeStore) {
  return function (context) {
    var store = makeStore(context);

    if (!isClient()) {
      if (isAppContext(context)) {
        context = context.ctx;
      }

      if (isSsrContext(context)) {
        // Create StateCookies object and smuggle it to the middleware via the `SERVE_COOKIES` action
        store.dispatch({
          type: SERVE_COOKIES,
          payload: new StateCookies(context)
        });
      }
    }

    return store;
  };
};
/**
 * A Redux middleware that syncs user-defined subtrees of the Redux state with cookies – on the
 * server and on the client. One cookie is used per state subtree and the serialized state is
 * compressed using [lz-string](https://github.com/pieroxy/lz-string). The subtree paths, cookie
 * names, and cookie options are configured via a {@link NextReduxCookieMiddlewareConfig} object.
 */

var nextReduxCookieMiddleware = function nextReduxCookieMiddleware(config) {
  return function (store) {
    var subtrees = processMiddlewareConfig(config);
    var cookieNames = subtrees.map(function (subtree) {
      return subtree.cookieName;
    });
    var cookies;

    if (isClient()) {
      cookies = new StateCookies();
      cookies.setAllNames(cookieNames);
    } // On the server, we have to intercept the `SERVE_COOKIES` action to get the `cookies` object
    // (we cannot directly set a property on the store, sadly, since the middleware does not have
    // direct access to the store object).


    return function (next) {
      return function (action) {
        switch (action.type) {
          case SERVE_COOKIES:
            {
              // Handle the SERVE_COOKIES action (server-only):
              cookies = action.payload;
              cookies.setAllNames(cookieNames); // Console.log("Cookies received by middleware");
              // We have access to the client's cookies now. Now we need to hydrate the store with their
              // state.

              var allCookies = cookies.getAll(); // Console.log("Triggering initial HYDRATE");

              store.dispatch({
                type: HYDRATE,
                payload: walkState(subtrees, function (subtree) {
                  return allCookies[subtree.cookieName];
                }, {})
              }); // We're done handling the action here (without calling `next()`).

              return;
            }

          case HYDRATE:
            if (isClient()) {
              // Intercept next-redux-wrapper's HYDRATE action on the client
              var _allCookies = cookies.getAll();

              action.payload = walkState(subtrees, function (subtreeConfig) {
                if (subtreeConfig.ignoreStateFromStaticProps) {
                  // `action.payload` holds the incoming server state. We overwrite that state with the
                  // state from the cookies: If the incoming state is from getServerSideProps, the
                  // cookies have also been updated to that state. If the incoming state is from
                  // getStaticProps, the cookies have remained unchanged and hence the server's state is
                  // ignored.
                  return _allCookies[subtreeConfig.cookieName];
                }
              }, action.payload);
            }

          // Fall through to default handling of the `HYDRATE` action

          default:
            {
              var oldState = store.getState();
              var result = next(action);
              var newState = store.getState(); // If cookies are available (which is not the case during `getStaticProps()`), write the
              // new state into cookies wherever necessary

              if (cookies) {
                walkState(subtrees, function (subtreeConfig, oldSubtreeState, newSubtreeState) {
                  if (!isEqual(oldSubtreeState, newSubtreeState)) {
                    var cookieName = subtreeConfig.cookieName,
                        cookieOptions = _objectWithoutPropertiesLoose(subtreeConfig, ["cookieName", "subtree", "ignoreStateFromStaticProps"]);

                    cookies.set(cookieName, newSubtreeState, cookieOptions);
                  }
                }, oldState, newState);
              }

              return result;
            }
        }
      };
    };
  };
};

export { SERVE_COOKIES, nextReduxCookieMiddleware, wrapMakeStore };
//# sourceMappingURL=next-redux-cookie-wrapper.esm.js.map
